@@ -5,11 +5,12 @@ cli
 click.group(name='main')
 
 '''
-
+import os
 import click
 from click.termui import prompt
-from clslq.clslq_pip import pip
-from clslq.clslq_venv import venv
+from clslq.cli_clslq_pip import pip
+from clslq.cli_clslq_venv import venv
+from clslq.cli_clslq_notion import notion
 
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 
@@ -21,7 +22,7 @@ def main():
     CLSLQ is a python library and command toolsets of Connard.
     Most of the contents are written in progress of python learning.
     CLSLQ include some quick-start python programming functions, wrappers and tools.
-    For more information, please contact [admin@lovelacelee.com].
+    For more information, please contact [lovelacelee@gmail.com].
     Documents available on https://clslq.readthedocs.io/.
     MIT License Copyright (c) Connard Lee.
     """
@@ -30,6 +31,7 @@ def main():
 
 main.add_command(pip, name='pip')
 main.add_command(venv, name='venv')
+main.add_command(notion, name='notion')
 
 if __name__ == '__main__':
     main()
